@@ -16,17 +16,19 @@ use frontend::{
         prelude::{GroupExt, WidgetBase, WidgetExt, WindowExt},
         window,
     },
-    get_frame_time, get_seconds_since_midnight, DpiScaling, Signal,
+    get_frame_time, DpiScaling, Signal, Timer,
 };
 use std::{cell::RefCell, rc::Rc};
 use std::{sync::Arc, time::Instant};
 
+const INTEGRATION_NAME: &str = "egui + fltk + wgpu-backend";
+
 fn main() {
     let a = app::App::default();
     let mut window = window::Window::default()
         .with_size(600, 800)
         .center_screen();
-    window.set_label("hello");
+    window.set_label("Demo Window");
     window.make_resizable(true);
     window.end();
     window.show();
@@ -87,7 +89,8 @@ fn main() {
             | Event::MouseWheel
             | Event::Resize
             | Event::Move
-            | Event::Drag => {
+            | Event::Drag
+            | Event::Focus => {
                 let mut handled = false;
                 // Using "if let ..." for safety.
                 if let Ok(mut state) = state.try_borrow_mut() {
@@ -110,7 +113,7 @@ fn main() {
     let egui_ctx = Rc::new(RefCell::new(CtxRef::default()));
     let repaint_signal = Arc::new(Signal::default());
     let start_time = Instant::now();
-    let mut app_output = AppOutput::default();
+    let app_output = Rc::new(RefCell::new(AppOutput::default()));
 
     // Redraw window while being resized (required on windows platform).
     window.draw({
@@ -121,6 +124,7 @@ fn main() {
         let device = device.clone();
         let queue = queue.clone();
         let demo_app = demo_app.clone();
+        let app_output = app_output.clone();
         move |window| {
             // And here also using "if let ..." for safety.
             if let Ok(mut state) = state.try_borrow_mut() {
@@ -130,17 +134,17 @@ fn main() {
                         let mut egui_ctx = egui_ctx.borrow_mut();
                         let mut device = device.borrow_mut();
                         let mut queue = queue.borrow_mut();
+                        let mut app_output = app_output.borrow_mut();
                         {
                             // Begin frame
                             let frame_time = get_frame_time(start_time);
-                            let seconds_since_midnight = get_seconds_since_midnight();
                             let mut frame = FrameBuilder {
                                 info: IntegrationInfo {
                                     web_info: None,
                                     cpu_usage: Some(frame_time),
-                                    seconds_since_midnight: Some(seconds_since_midnight),
                                     native_pixels_per_point: Some(state.pixels_per_point),
                                     prefer_dark_mode: None,
+                                    name: INTEGRATION_NAME,
                                 },
                                 tex_allocator: &mut painter.render_pass,
                                 output: &mut app_output,
@@ -174,23 +178,25 @@ fn main() {
         }
     });
 
+    let mut timer = Timer::new(1);
+
     while a.wait() {
         let mut state = state.borrow_mut();
         let mut painter = painter.borrow_mut();
         let mut egui_ctx = egui_ctx.borrow_mut();
         let mut device = device.borrow_mut();
         let mut queue = queue.borrow_mut();
+        let mut app_output = app_output.borrow_mut();
         {
             // Begin frame
             let frame_time = get_frame_time(start_time);
-            let seconds_since_midnight = get_seconds_since_midnight();
             let mut frame = FrameBuilder {
                 info: IntegrationInfo {
                     web_info: None,
                     cpu_usage: Some(frame_time),
-                    seconds_since_midnight: Some(seconds_since_midnight),
                     native_pixels_per_point: Some(state.pixels_per_point),
                     prefer_dark_mode: None,
+                    name: INTEGRATION_NAME,
                 },
                 tex_allocator: &mut painter.render_pass,
                 output: &mut app_output,
@@ -214,15 +220,16 @@ fn main() {
 
         let window_resized = state.window_resized();
         if window_resized {
-            window.clear_damage()
+            window.clear_damage();
         }
 
-        if output.needs_repaint || window_resized {
+        // Make sure timer.elapsed() in the last order.
+        if output.needs_repaint || window_resized || timer.elapsed() {
             state.fuse_output(&mut window, &output);
             let clipped_mesh = egui_ctx.tessellate(shapes);
             let texture = egui_ctx.texture();
             painter.paint_jobs(&mut device, &mut queue, &mut state, clipped_mesh, texture);
-            app::awake()
         }
+        app::awake();
     }
 }
